/***********************************************************************/
/*                                                                     */
/*  FILE        :cbm.c                                                 */
/*  DATE        :26.12.08                                              */
/*  COPYRIGHT   :Copyright (C) 2008 by Thomas Hochgtz                 */
/*                                                                     */
/***********************************************************************/
              
#include "cbm.h"
#include "r8c.h"

struct TIMEOUTS  timeout;

			      
void		InitCBM()
{
	LED_0_D = 1;
	LED_1_D = 1;
	LED_2_D = 1;
	LED_3_D = 1;
	
	LED_0 = 0;
	LED_1 = 0;
	LED_2 = 0;
	LED_3 = 0;

	CBM_DTA_D = 1;
	CBM_RST_D = 1;
	CBM_ATN_D = 1;
	CBM_CLK_D = 1;

	CBM_DTA_OUT( 0 );
	CBM_RST_OUT( 0 );
	CBM_ATN_OUT( 0 );
	CBM_CLK_OUT( 0 );
	
	ResetTimeouts();
}


void	ResetCBM()
{
	CBM_DTA_OUT( 0 );
	CBM_RST_OUT( 0 );
	CBM_ATN_OUT( 0 );
	CBM_CLK_OUT( 0 );
	
	LED_0 = 0;
	LED_1 = 0;
	LED_2 = 0;
	LED_3 = 0;
}


void	ResetTimeouts()
{
	timeout.Ts = 70;
	timeout.Tv = 60;
	timeout.Tbb = 100; 		// Tbb or Tr
	timeout.Tat = 1000; 	// ReadyToSend
	timeout.Tne = 40;		// ReadyToSend
}


void	SetTimeouts( int type, unsigned char value )
{
	if( TIMEOUT_TS == type )  		timeout.Ts = value;
	else if( TIMEOUT_TV == type )  	timeout.Tv = value;
	else if( TIMEOUT_TBB == type ) 	timeout.Tbb = value; 		// Tbb or Tr
	else if( TIMEOUT_TAT == type ) 	timeout.Tat = (unsigned short)value*10; 	// ReadyToSend
	else if( TIMEOUT_TNE == type ) 	timeout.Tne = value;		// ReadyToSend
}


void	SendByte( unsigned char data )
{
	unsigned short i = 0;
	for( ; i < 8; i++ )
	{
		CBM_DTA_BIT_OUT( (data >> i) & 1 );
		Delay( timeout.Ts ); // Ts
		CBM_CLK_OUT( 0 );
		Delay( timeout.Tv ); // Tv
		CBM_CLK_OUT( 1 );
		CBM_DTA_OUT( 0 );
	}
}


unsigned char	RecvByte( unsigned char* pData )
{
	unsigned char data = 0;
	unsigned short i = 0;
	unsigned short w = 0;
	for( ; i < 8; i++ )
	{
		for( w = 0; w < 260; w++ ) { // Tbb
			if( !CBM_CLK_IN ) break;
			asm("nop");
		}
		if( w >= 260 ) {
			return RET_TIMEOUT;
		}
		
		data |= CBM_DTA_BIT_IN << i;
		
		for( w = 0; w < 160; w++ ) {
			if( CBM_CLK_IN ) break;
			asm("nop");
		}
		if( w >= 160 ) {
			return RET_TIMEOUT;
		}
	}
	*pData = data;
	return RET_OK;
}


unsigned char	SendAckFrame()
{
	Delay( 20 );  // Tf
	CBM_DTA_OUT( 1 );
	Delay( 30 ); // Delay( 100 ); // Tbb
}


unsigned char	RecvAckFrame()
{
	unsigned short w = 0;
	for( w = 0; w < 1000; w++ ) {  // Tf
		if( CBM_DTA_IN ) break;
		asm("nop");
	}
	if( w >= 1000 ) {
		return RET_TIMEOUT;  // frame error
	}
	Delay( timeout.Tbb ); // Tbb or Tr
	return RET_OK;
}


unsigned char	ReadyToSend( unsigned char EOI, unsigned char startATN )
{
	unsigned short w = 0;
	
	if( startATN )
	{
		CBM_CLK_OUT( 1 );
		CBM_DTA_OUT( 0 );
		
		Delay( timeout.Tat ); // Tat
		if( !CBM_DTA_IN ) {
			return RET_TIMEOUT;
		}
	}
	
	CBM_CLK_OUT( 0 );
	
	for( w = 0; w < 10; w++ )	// Th
	{
		if( !CBM_DTA_IN ) break;
		asm("nop");
	}
	if( w >= 10 ) {
		return RET_TIMEOUT_POLL;
	}
	
	if( EOI ) {
		return RET_EOI;
	}
	
	Delay( timeout.Tne ); // Tne
	CBM_CLK_OUT( 1 );
	return RET_OK;
}


unsigned char	ReadyToRecv()
{
	unsigned short w = 0;
		
	// server ready to send
	for( w = 0; w < 10; w++ )
	{
		if( !CBM_CLK_IN ) break;
		asm("nop");
	}
	if( w >= 10 ) {
		return RET_TIMEOUT_POLL;
	}
	
	CBM_DTA_OUT( 0 );

	// check all clients have released
	for( w = 0; w < 1000; w++ )
	{
		if( !CBM_DTA_IN ) break;
		asm("nop");
	}
	if( w >= 1000 ) {
		return RET_TIMEOUT;
	}

	for( w = 0; w < 200; w++ )	// Tne
	{
		if( CBM_CLK_IN ) break;
		asm("nop");
	}
	if( w >= 200 ) {
		return RET_EOI;
	}
	return RET_OK;
}


unsigned char	RecvAckEOI()
{
	unsigned short w = 0;
	for( w = 0; w < 500; w++ )
	{
		if( CBM_DTA_IN ) break;
		asm("nop");
	}
	if( w >= 500 ) {
		return RET_TIMEOUT;
	}
	
	for( w = 0; w < 120; w++ )
	{
		if( !CBM_DTA_IN ) break;
		asm("nop");
	}
	if( w >= 120 ) {
		return RET_TIMEOUT;
	}
	Delay( 30 );
	CBM_CLK_OUT( 1 );
	return RET_OK;
}


unsigned char	SendAckEOI()
{
	unsigned short w = 0;
	CBM_DTA_OUT( 1 );
	Delay( 80 );	// Tei
	CBM_DTA_OUT( 0 );
	
	for( w = 0; w < 80; w++ )
	{
		if( CBM_CLK_IN ) break;
		asm("nop");
	}
	if( w >= 80 ) {
		return RET_TIMEOUT;
	}

	return RET_OK;
}


unsigned char	TurnSender()
{
	unsigned short w = 0;

	Delay( 30 ); // Ttk
	
	CBM_DTA_OUT( 1 );
	CBM_CLK_OUT( 0 );

	for( w = 0; w < 500; w++ )
	{
		if( !CBM_DTA_IN ) break;
		asm("nop");
	}
	if( w >= 500 ) {
		return RET_TIMEOUT;
	}
	return RET_OK;
}


unsigned char	TurnReceiver()
{
	unsigned short w = 0;
	
	for( w = 0; w < 120; w++ ) // Ttk
	{
		if( !CBM_CLK_IN ) break;
		asm("nop");
	}
	if( w >= 120 ) {
		return RET_TIMEOUT;
	}
	
	CBM_CLK_OUT( 1 );
	CBM_DTA_OUT( 0 );
	
	Delay( 80 ); // Tda
	
	return RET_OK;
}


unsigned char	WaitReleasedATN()
{
	unsigned short w = 0;
	
	for( w = 0; w < 80; w++ )
	{
		if( !CBM_ATN_IN ) break;
		asm("nop");
	}
	if( w >= 80 ) {
		return RET_TIMEOUT;
	}
	
	return RET_OK;
}


unsigned char	WaitReleasedReset()
{
	unsigned short w = 0;
	
	for( w = 0; w < 50; w++ )
	{
		if( !CBM_RST_IN ) break;
		asm("nop");
	}
	if( w >= 50 ) {
		return RET_TIMEOUT_POLL;
	}
	
	Delay( 500 );
	
	return RET_OK;
}


void	ZeroSender()
{
	CBM_DTA_OUT( 0 );
	CBM_CLK_OUT( 1 );
}


void	ZeroReceiver()
{
	CBM_CLK_OUT( 0 );
	CBM_DTA_OUT( 1 );
}


void	OffSender()
{
	Delay( 60 );
	CBM_DTA_OUT( 0 );
	CBM_CLK_OUT( 0 );
}


void	OffReceiver()
{
	Delay( 60 );
	CBM_CLK_OUT( 0 );
	CBM_DTA_OUT( 0 );
}
