/***********************************************************************/
/*                                                                     */
/*  FILE        :cbm.h                                                 */
/*  DATE        :26.12.08                                              */
/*  COPYRIGHT   :Copyright (C) 2008 by Thomas Hochgtz                 */
/*                                                                     */
/***********************************************************************/
    
#ifndef _CBM_H_
#define _CBM_H_     


#include "sfr_r813.h"  

             

//=============== MAIN =============
#define		LED_0			p1_0
#define		LED_1			p1_1
#define		LED_2			p1_2
#define		LED_3			p1_3

#define		LED_0_D			pd1_0
#define		LED_1_D			pd1_1
#define		LED_2_D			pd1_2
#define		LED_3_D			pd1_3

#define     CBM_DTA_IN 		!p1_7
#define     CBM_DTA_BIT_IN 	p1_7					// data bits are reverse logic
#define     CBM_RST_IN		!p3_2
#define		CBM_ATN_IN		!p3_3
#define		CBM_CLK_IN		!p4_5

#define     CBM_DTA_OUT( b )		p0_4 = !(b);
#define     CBM_DTA_BIT_OUT( b )	p0_4 = (b);		// data bits are reverse logic
#define     CBM_RST_OUT( b )		p0_5 = !(b);
#define		CBM_ATN_OUT( b )		p0_6 = !(b);
#define		CBM_CLK_OUT( b )		p0_7 = !(b);

#define     CBM_DTA_D				pd0_4
#define     CBM_RST_D				pd0_5
#define		CBM_ATN_D				pd0_6
#define		CBM_CLK_D				pd0_7

#define     INIT_UART				InitUART0()
#define     SEND_UART( data )		SendUART0( data )
#define     SEND_TRY_UART( data )	SendTryUART0( data )
#define     RECV_UART				ReceiveUART0()
#define     RECV_TRY_UART			ReceiveTryUART0()
#define     RECV_CHECK_UART			CheckReceiveUART0


//=============== FUNCTIONS =============
void					InitCBM();

void					ResetCBM();
void					ResetTimeouts();
void					SetTimeouts( int type, unsigned char value );

void					SendByte( unsigned char data );
unsigned char			RecvByte( unsigned char* pData );

unsigned char			SendAckFrame();
unsigned char			RecvAckFrame();

unsigned char			ReadyToSend( unsigned char EOI, unsigned char startATN );
unsigned char			ReadyToRecv();

unsigned char			SendAckEOI();
unsigned char			RecvAckEOI();

unsigned char			TurnSender();
unsigned char			TurnReceiver();

void					ZeroSender();
void					ZeroReceiver();
void					OffSender();
void					OffReceiver();

unsigned char			WaitReleasedATN();
unsigned char			WaitReleasedReset();


enum RETCODE
{
	RET_OK,
	RET_ERROR,
	RET_TIMEOUT,
	RET_TIMEOUT_POLL,
	RET_EOI,
};


enum PRGCODE
{
	PRG_LISTEN			= 0x20,
	PRG_LISTEN_LAST		= 0x3E,
	PRG_TALK			= 0x40,
	PRG_TALK_LAST		= 0x5E,
	PRG_UNLISTEN		= 0x3F,
	PRG_UNTALK			= 0x5F,

	PRG_CH_DATA			= 0x60,
	PRG_CH_DATA_LAST	= 0x6F,
	PRG_CH_CLOSE		= 0xE0,
	PRG_CH_CLOSE_LAST	= 0xEF,
	PRG_CH_OPEN			= 0xF0,
	PRG_CH_OPEN_LAST	= 0xFF,
};



struct TIMEOUTS
{
	unsigned char	Ts;
	unsigned char	Tv;
	unsigned char 	Tbb;
	unsigned short 	Tat;
	unsigned char	Tne;
};


extern struct TIMEOUTS	timeout;


enum PL_TIMEOUTS
{
	TIMEOUT_ALL,
	TIMEOUT_TS,
	TIMEOUT_TV,
	TIMEOUT_TBB,
	TIMEOUT_TAT,  // *10
	TIMEOUT_TNE,
};



#endif


