/***********************************************************************/
/*                                                                     */
/*  FILE        :proto_device.c                                        */
/*  DATE        :26.12.08                                              */
/*  COPYRIGHT   :Copyright (C) 2008 by Thomas Hochgtz                 */
/*                                                                     */
/***********************************************************************/
              
#include "proto_device.h"
#include "proto.h"
#include "cbm.h"
#include "r8c.h"


#define DO_ERROR( err ) { sett.error = err; LED_3 = 1; SEND_UART(PC_ERROR); SEND_UART(err); }

			      


void	ResetATN()
{
	sett.revert = 0;
	sett.dorevert = 0;
	SwitchDirection( 0 );
	sett.EOI = 0;
	sett.SET_EOI = 0;
	sett.SET_LAST = 0;
	sett.ATN = 0;
	sett.data_send = 0;
	ZeroReceiver();
}



void	CheckDevice()
{
	if( CBM_RST_IN )
	{
		SEND_UART( PC_RST_ON );
		ResetProto();
		while( WaitReleasedReset() == RET_TIMEOUT_POLL )
		{
			ReceiveUART( 0 );
		}
		ZeroReceiver();
		SEND_UART( PC_RST_OFF );
	}
	if( CBM_ATN_IN )
	{
		if( sett.ATN == 0 )
		{
			ResetATN();
			sett.ATN = 1;
			SEND_UART( PC_ATN_ON );
		}
	}
	else
	{
		if( sett.ATN )
		{
			sett.ATN = 0;
			sett.ATNcount = 0;
			SEND_UART( PC_ATN_OFF );
		}
		if( sett.dorevert )
		{
			SwitchDirection( 1 );
			sett.dorevert = 0;
			if( RET_TIMEOUT == TurnReceiver() ) {
				DO_ERROR( PCERR_TURN );
			}
			sett.revert = 1;
		}
	}
}


void	ReceiverDevice()
{
	unsigned char ret;
	unsigned char data;
	
	if( (!sett.select && !sett.ATN) || !CBM_DTA_IN ) {
		ReceiveUART( 0 );
		Delay( 20 );
		return;
	}
	
	ret = ReadyToRecv();
	
	if( ret == RET_TIMEOUT_POLL )
	{
		ReceiveUART( 0 );
	}
	else if( ret == RET_TIMEOUT )
	{
		DO_ERROR( PCERR_READY_TO_RECV );
	}
	else
	{
		if( ret == RET_EOI )
		{
			if( RET_TIMEOUT == SendAckEOI() ) {
				DO_ERROR( PCERR_ACK_EOI );
			}
			sett.EOI = 1;
			SEND_UART( PC_EOI );
		}
		
		ret = RecvByte( &data );
		if( ret == RET_OK )
		{
			SendAckFrame();
			
			SEND_UART( PC_DATA );
			SEND_UART( data );
			
			if( sett.ATN )
			{
				if( 0 == sett.ATNcount ) {
					CommandReceivedDevice( data );
				}
				sett.ATNcount++;
			}
			if( sett.EOI )
			{
				ZeroReceiver();
				sett.EOI = 0;
				SwitchDirection( 2 );
			}
		}
		else
		{
			DO_ERROR( PCERR_RECV_BYTE );
		}
	}
}


void	CommandReceivedDevice( unsigned char data )
{
	if( data >= PRG_LISTEN && data <= PRG_LISTEN_LAST )
	{
		if( (data - PRG_LISTEN) == sett.dev_addr )
		{
			sett.select = 1;
			LED_1 = 1;
			LED_2 = 1;
		}
	}
	else if( data >= PRG_TALK && data <= PRG_TALK_LAST )
	{
		if( (data - PRG_TALK) == sett.dev_addr )
		{
			sett.select = 1;
			LED_1 = 1;
			sett.dorevert = 1;
		}
	}
	else if( data == PRG_UNLISTEN )
	{
		ZeroReceiver();
		sett.EOI = 0;
		sett.select = 0;
		LED_1 = 0;
		LED_2 = 0;
	}
	else if( data == PRG_UNTALK )
	{
		ZeroReceiver();
		sett.EOI = 0;
		sett.select = 0;
		LED_1 = 0;
	}
}



void	SenderDevice()
{
	unsigned char ret;
	unsigned char data;
		
				
	if( !sett.data_send )
	{
		sett.data = ReceiveUART( 1 );
		sett.data_send = 1;
	}
	
	ret = ReadyToSend( sett.SET_EOI, 0 );
		
	if( ret == RET_TIMEOUT_POLL )
	{
	}
	else if( ret == RET_EOI || ret == RET_OK )
	{
		if( ret == RET_EOI )
		{			
			sett.SET_EOI = 0;
			sett.SET_LAST = 1;
			if( RET_TIMEOUT == RecvAckEOI() ) {
				DO_ERROR( PCERR_ACK_EOI );
			}
		}

		SendByte( sett.data );
		sett.data_send = 0;
		
		if( RET_TIMEOUT == RecvAckFrame() ) {
			DO_ERROR( PCERR_ACK_FRAME );
		}
		
		if( sett.SET_LAST )
		{
			sett.SET_LAST = 0;
			SwitchDirection( 2 );
		}
	}
	else
	{
		DO_ERROR( PCERR_READY_TO_SEND );
	}
}









