/***********************************************************************/
/*                                                                     */
/*  FILE        :rc8.c                                                 */
/*  DATE        :26.12.08                                              */
/*  COPYRIGHT   :Copyright (C) 2008 by Thomas Hochgtz                 */
/*                                                                     */
/***********************************************************************/
                  
#include "sfr_r813.h"  
#include "r8c.h"


void EnableMainClock()
{
   /*-------------------------------------------------
    -  Change on-chip oscillator clock to Main clock  -
    -------------------------------------------------*/

    prc0 = 1;           	/* Protect off */
    cm13 = 1;           	/* Xin Xout */
    cm15 = 1;           	/* XCIN-XCOUT drive capacity select bit : HIGH */
    cm05 = 0;           	/* Xin on */
    cm16 = 0;           	/* Main clock = No division mode */
    cm17 = 0;
    cm06 = 0;           	/* CM16 and CM17 enable */
 	asm("nop");         	/* Waiting for stable of oscillation */
    asm("nop");
    asm("nop");
    asm("nop");
    ocd2 = 0;           	/* Main clock change */
    prc0 = 0;           	/* Protect on */
}


void EnableInterrupt()
{
    asm( "FSET I");	  /* Enable interrupt */
}


void Delay( unsigned int micros )
{
	unsigned int i;
	for (i = 0; i < micros; i++)
	{
		asm("nop");
	}
}


void DelayMs( unsigned int millis )
{
	unsigned int i;
	for (i = 0; i < millis; i++)
	{
		Delay( 1000 );
	}
}


void InitUART0()
{
    p1 = p1 | 0x10;   // TxD0 port output 
    pd1 = pd1 | 0x10; // TxD0 port direction = output 
    pd1 = pd1 & 0xdf; // RxD0 port direction = input 
    u0mr = 0x05;      // UART0 transmit/receive mode 
    u0c0 = 0x00;      // UART0 transmit/receive control 
    u0rrm = 0;        // Continuous receive mode disabled 
    u0brg = 32-1;     // 38400 baud @20MHz 
    re_u0c1 = 1;      // Reception enabled 
}


void SendUART0( unsigned char data )
{
	while( ti_u0c1 == 0 );	//Wait for transmission buffer emty
	u0tbl = data;       	  // Set transmission data 
  	te_u0c1 = 1;        	  // Transmission enabled 
}

char SendTryUART0( unsigned char data )
{
	if( ti_u0c1 == 0 ) {
		return -1;
	}
	u0tbl = data;       	  // Set transmission data 
  	te_u0c1 = 1;        	  // Transmission enabled 
	return 0;
}


unsigned char ReceiveUART0()
{
	unsigned char data;
	unsigned char dummy;
	while( ir_s0ric == 0 ); 	//Wait for received data
  	ir_s0ric = 0;    		      //Clear serial reception flag 
	data = u0rbl;     		    // Get reception data 
	dummy = u0rbh;     		    // Get error 
	re_u0c1 = 1;      		    // Reception enabled 
	return data;
}

unsigned char ReceiveTryUART0()
{
	unsigned char data;
	unsigned char dummy;
	if( ir_s0ric == 0 ) {
		return -1;
	}
  	ir_s0ric = 0;    		      //Clear serial reception flag 
	data = u0rbl;     		    // Get reception data 
	dummy = u0rbh;     		    // Get error 
	re_u0c1 = 1;      		    // Reception enabled 
	return data;
}


void InitUART1()
{
    p0_0 = 1;   	  // TxD1 port output 
    pd0_0 = 1;        // TxD1 port direction = output 
    u1mr = 0x05;      // UART1 transmit/receive mode 
    u1c0 = 0x00;      // UART1 transmit/receive control 
    //u1rrm = 0;        // Continuous receive mode disabled 
    u1brg = 32-1;     // 38400 baud @20MHz 
    re_u1c1 = 1;      // Reception enabled 
}


void SendUART1( unsigned char data )
{
	while( ti_u1c1 == 0 );	//Wait for transmission buffer emty
	u1tbl = data;       	  // Set transmission data 
  	te_u1c1 = 1;        	  // Transmission enabled 
}

char SendTryUART1( unsigned char data )
{
	if( ti_u1c1 == 0 ) {
		return -1;
	}
	u1tbl = data;       	  // Set transmission data 
  	te_u1c1 = 1;        	  // Transmission enabled 
	return 0;
}


unsigned char ReceiveUART1()
{
	unsigned char data;
	unsigned char dummy;
	while( ir_s1ric == 0 ); 	//Wait for received data
  	ir_s1ric = 0;    		      //Clear serial reception flag 
	data = u1rbl;     		    // Get reception data 
	dummy = u1rbh;     		    // Get error 
	re_u1c1 = 1;      		    // Reception enabled 
	return data;
}

unsigned char ReceiveTryUART1()
{
	unsigned char data;
	unsigned char dummy;
	if( ir_s1ric == 0 ) {
		return -1;
	}
  	ir_s1ric = 0;    		      //Clear serial reception flag 
	data = u1rbl;     		    // Get reception data 
	dummy = u1rbh;     		    // Get error 
	re_u1c1 = 1;      		    // Reception enabled 
	return data;
}
